/*
  This program is free software published under the
  terms of the GNU Lesser General Public License.
  http://www.gnu.org/copyleft/lesser.html    
  This program is distributed in the hope that it will
  be useful - WITHOUT ANY WARRANTY; without even the
  implied warranty of MERCHANTABILITY or FITNESS FOR A
  PARTICULAR PURPOSE.
*/ 

// xxxxxxxxxxxxxxx              SIMPLE js SHAPES LIBRARY     xxxxxxxxxxxxxxxxxxxxx


/* draw full circle or circle segment
  for parameters see: https://www.w3schools.com/tags/canvas_arc.asp 
  col = color (fillStyle)
*/  
function circle(ctx, x, y, r, sAngle, eAngle, clock, col)
{
  ctx.beginPath(); 
  ctx.arc(x, y, r, sAngle, eAngle, clock);  //ctx.arc(x,y,r,sAngle,eAngle,counterclockwise);            
  ctx.fillStyle= col;       
  ctx.fill();  
}

/* create polygon with equal line length
  ctx = canvas context instance, 
  x = x-center of polygon,
  y = y-center of polygon,
  r = radius,
  lines = line count,
  col = color (fillStyle) 
*/  
function polygon(ctx, x, y, r, lines, col)
{
  ctx.beginPath();
  for (var j= 1; j<=lines; j++) 
  {            
    var px = x + r * Math.cos(j*2*Math.PI/lines);
    var py = y + r * Math.sin(j*2*Math.PI/lines);
    j===1 ? ctx.moveTo(px,py) : ctx.lineTo(px,py);                     
  }    
  ctx.fillStyle= col;     
  ctx.fill();
}

/* draw square => rect with width= height;
    ctx = canvas (object) context,
    x = x-center,
    y= y-center,
    r = radius,
    col = color (fillStyle) 
*/
function square(ctx, x, y, r, col)
{  
  ctx.beginPath();  
  ctx.rect(x-r, y-r, 2*r, 2*r);
  ctx.fillStyle= col;     
  ctx.fill();
}

/* draw rectangle on canvas
    ctx = canvas (object) context,
    x = x-center,
    y= y-center,
    r = radius,
    col = color (fillStyle) 
    if (hor == true) => horizontal line is longer then vertical;  else  vertical line is longer then horizontal  
*/    
function rectangle(ctx, x, y, r, col, hor)
{    
  ctx.beginPath();                      
  if (hor) // horizontal line is longer then vertical
  {
    ctx.moveTo(x-r,y+r/2);
    ctx.lineTo(x-r,y-r/2);
    ctx.lineTo(x+r,y-r/2);
    ctx.lineTo(x+r,y+r/2);
    ctx.lineTo(x-r, y+r/2);   
  }
  else  
  {   // vertical line is longer then horizontally
    ctx.moveTo(x-r/2,y+r);
    ctx.lineTo(x-r/2,y-r);
    ctx.lineTo(x+r/2,y-r);
    ctx.lineTo(x+r/2,y+r);
    ctx.lineTo(x-r/2,y+r);      
  }    
  ctx.fillStyle= col;     
  ctx.fill();
}


// from source: https://gist.github.com/devyn/243740
/*
 * Draw a rounded rectangle path.
 *
 * x, y: position of the rectangle
 *
 * w, h: width and height of the rectangle
 *
 * r (Integer): the radius of each corner
 * r (Array):   an array of radii like so:
 *                [topLeft, topRight, bottomRight, bottomLeft]
 *  added par:
 *  ctx : canvas context instance of CanvasRenderingContext2D.prototype
 *  col:  color
*/ 
function roundRect(ctx, x, y, w, h, r, col) 
{
  with (ctx) 
  {
    if (r instanceof Array) 
    {
      var r1 = r[0];
      var r2 = r[1];
      var r3 = r[2];
      var r4 = r[3];
    } else 
    {
      var r1 = r;
      var r2 = r;
      var r3 = r;
      var r4 = r;
    }
    beginPath();
    moveTo(x, y+r1);
    quadraticCurveTo(x, y, x+r1, y);
    lineTo(x+w-r2, y);
    quadraticCurveTo(x+w, y, x+w, y+r2);
    lineTo(x+w, y+h-r3);
    quadraticCurveTo(x+w, y+h, x+w-r3, y+h);
    lineTo(x+r4, y+h);
    quadraticCurveTo(x, y+h, x, y+h-r4);
    lineTo(x, y+r1);
    fillStyle= col;
    fill(); 
    //strokeStyle= col;   stroke(); // line color (outer border line color)     
  }   
};

 
/* draw Trapezoid  
    ctx = canvas (object) context,
    x = x-center,
    y= y-center,
    r = radius,
    col = color (fillStyle) 
    base_down = longer side (line)
*/
function trapezoid(ctx, x, y, r, col, base_down)
{  
  ctx.beginPath();                    
  if (base_down)  // trapezoid (base down)
  {
    ctx.moveTo(x-r,y+r);
    ctx.lineTo(x-r/2,y-r);
    ctx.lineTo(x+r/2,y-r);
    ctx.lineTo(x+r,y+r);
    ctx.lineTo(x-r,y+r);    
  }
  else   // trapezoid (base up)
  {
    ctx.moveTo(x-r/2,y+r);
    ctx.lineTo(x-r,y-r);
    ctx.lineTo(x+r,y-r);
    ctx.lineTo(x+r/2,y+r);
    ctx.lineTo(x-r/2,y+r); 
  }  
  ctx.fillStyle= col;     
  ctx.fill();
}

/* draw Parallelogram 
    ctx = canvas (object) context,
    x = x-center,
    y= y-center,
    r = radius,
    col = color (fillStyle) 
    if (base == true) => down-left direction. LTR direction: DOWN-UP oriented   
    if (!base) =>  up-down direction. LTR direction: UP-DOWN oriented  
*/    
function paralelogram(ctx, x, y, r, col, base)
{  
  ctx.beginPath();
  var k= 0.25;  // empirically
  if (base)  
  {   // Parallelogram (down-left direction). LTR direction: DOWN-UP oriented                
    ctx.moveTo(x-r,y+k*r);
    ctx.lineTo(x+k*r,y-r);
    ctx.lineTo(x+r,y-k*r); 
    ctx.lineTo(x-k*r,y+r);
    ctx.lineTo(x-r,y+k*r);    
  }
  else  
  {   // Parallelogram (up-down direction). LTR direction: UP-DOWN oriented
    ctx.moveTo(x-r,y-k*r);
    ctx.lineTo(x-k*r,y-r);
    ctx.lineTo(x+r,y+k*r); 
    ctx.lineTo(x+k*r,y+r);
    ctx.lineTo(x-r,y-k*r);    
  }
  ctx.fillStyle= col;     
  ctx.fill();
}       


// mostly downloaded from: https://stackoverflow.com/questions/2172798/how-to-draw-an-oval-in-html5-canvas
function ellipse(ctx, x, y, w, h, col) 
{
  /**    
         * Kappa is the value which which to scale the curve handles when
         * drawing a circle with bezier curves.
         *
         * http://whizkidtech.redprince.net/bezier/circle/kappa/
        
        KAPPA: 4 * (sqrt(2) - 1) / 3,
        KAPPA: 4 * (sqrt(2) - 1) / 3,
  */      
                //  0.552284 7498307936
  var kappa = (4 * (Math.sqrt(2) - 1) / 3),
  ox = (w / 2) * kappa, // control point offset horizontal
  oy = (h / 2) * kappa, // control point offset vertical
  xe = x + w,           // x-end
  ye = y + h,           // y-end
  xm = x + w / 2,       // x-middle
  ym = y + h / 2;       // y-middle

  ctx.beginPath();
  ctx.moveTo(x, ym);
  ctx.bezierCurveTo(x, ym - oy, xm - ox, y, xm, y);
  ctx.bezierCurveTo(xm + ox, y, xe, ym - oy, xe, ym);
  ctx.bezierCurveTo(xe, ym + oy, xm + ox, ye, xm, ye);
  ctx.bezierCurveTo(xm - ox, ye, x, ym + oy, x, ym);
  
  ctx.fillStyle= col;  
  ctx.fill(); 
  ctx.stroke();
}

/* draw star 
    ctx = canvas (object) context,
    x0 = x-center,
    y0= y-center,  
    numPoints = number of star blades,
    innRadius = inner radius,
    outRadius= outer radius
    col = color (fillStyle)   
*/    
function star(ctx, x0, y0, numPoints, innRadius, outRadius, col)
{  
  ctx.beginPath();
  ctx.moveTo(x0, y0-outRadius);    //ctx.moveTo(0, 0 - outRadius);
  for (var n = 1; n < numPoints * 2; n++) 
  {
    var radius = n % 2 === 0 ? outRadius : innRadius;
    var x = radius * Math.sin(n * Math.PI / numPoints);
    var y = -1 * radius * Math.cos(n * Math.PI / numPoints);
    ctx.lineTo(x0+x, y0+y);
  }    
  ctx.fillStyle= col;  
  ctx.fill();         
  ctx.closePath(); 
  
  ctx.strokeStyle= col;  // line color (outer border line color) 
  ctx.stroke(); 
  ctx.strokeStyle= "#000000";  // restore to black  
}


/* draw Cross 
    ctx = canvas (object) context,
    x = x-center,
    y= y-center,
    r = radius,  
    col = color,
    (ver == true)  => draw classic vertical cross; else draw diagonal cross (x) 
*/                                         
function cross(ctx, x, y, r, col, ver)
{
  ctx.beginPath();
  var _prev= ctx.lineWidth; // save 
  ctx.lineWidth= r/2;
  if (ver)
  {  // classic vertical cross
    ctx.moveTo(x-r,y);
    ctx.lineTo(x+r,y);
    ctx.moveTo(x,y-r);
    ctx.lineTo(x,y+r);    
  }
  else
  {  // diagonal cross (x)
    ctx.moveTo(x-r/2,y+r/2);
    ctx.lineTo(x+r/2,y-r/2);
    ctx.moveTo(x-r/2,y-r/2);
    ctx.lineTo(x+r/2,y+r/2);     
  }                          
  ctx.strokeStyle= col; 
  ctx.stroke(); 
  ctx.strokeStyle= "#000000";   // restore to black
  ctx.lineWidth= _prev; // restore    
}


/* draw Arrow
    ctx = canvas (object) context,
    x = x-center,
    y= y-center,
    r = radius, 
    col = color, 
    (par ===0) => vertical UP arrow
    (par===1)  =>  vertical DOWN arrow
    (par===2)  =>  horizontal LEFT arrow
    (par===3)  => horizontal RIGHT arrow 
*/
function arrow(ctx, x, y, r, col, par)
{    
  if (par < 0 || par > 3) return;   
  var k1=0.25*r,    k2= 0.75*r; 
  // init path (points) for different par argument
  var p=  [  
  // vertical UP arrow => p[0]
  [  
     x-k1,y+r, 
     x+k1,y+r, 
     x+k1,y-k1, 
     x+k2,y-k1, 
     x,y-r, 
     x-k2,y-k1, 
     x-k1,y-k1     
   ],                                          
  //  vertical DOWN arrow => p[1]
  [ 
      x-k1,y-r, 
      x-k1,y+k1, 
      x-k2,y+k1,    
      x,y+r, 
      x+k2,y+k1,
      x+k1,y+k1,
      x+k1,y-r     
   ],    
  //  horizontal LEFT arrow => p[2]
  [
      x+r, y+k1, 
      x-k1,y+k1, 
      x-k1,y+k2 ,   
      x-r,y,
      x-k1,y-k2,
      x-k1,y-k1,
      x+r,y-k1        
  ],        
  //  horizontal RIGHT arrow => p[3]
  [
     x-r, y+k1, 
     x+k1,y+k1, 
     x+k1, y+k2,    
     x+r,y, 
     x+k1,y-k2,
     x+k1,y-k1,
     x-r,y-k1          
  ]  
  ];
   
  ctx.beginPath();
  ctx.fillStyle= col; 
  for (var j=0; j<p[par].length; j=j+2)  {
    (j==0) ? ctx.moveTo(p[par][j],  p[par][j+1]) : ctx.lineTo(p[par][j],  p[par][j+1]);            
  }    
  ctx.closePath();
  ctx.fill();        
}


/* Plotting Archimedean spiral
  input arguments:
  ctx => canvas context instance of CanvasRenderingContext2D.prototype
  (x0,y0) => x, y center of Archimedean spiral
  r => outer_radius (outer edge of Archimedean_spiral)
  n_esc => invisible number of loops  [escaped spiral loops count]
  n_tot => total number of loops [spiral loops count]
  col => color of Archimedean_spiral
  example call: 6 loops are visible: spiral(ctx, x0, y0, r, 0, 6, col)
  example call: 6 loops total , first 3 are not visible: spiral(ctx, x0, y0, r, 3, 6, col)
*/
function spiral(ctx, x0, y0, r, n_esc, n_tot, col) 
{
 // r = a + b * angle 
 // a = inner radius from (x0,y0) = r_inner => start point of the spiral from (x0,y0)
 // b = gap = distance between each arm   
  if (r < 0 || n_esc >= n_tot) return;   // check arguments
  var gap = r/n_tot/(2*Math.PI); // gap = b = spacing (distance) between spiral lines        
  var STEPS = 60; // increasing this makes the curve smoother      
  var increment = 2*Math.PI/STEPS;		
  var theta = increment + (n_esc * 2*Math.PI);           
  ctx.beginPath();
  ctx.moveTo(x0+theta*Math.cos(theta)*gap, y0+theta*Math.sin(theta)*gap);  
  while(theta < n_tot*2*Math.PI) 
  {
     var x = x0 + theta * Math.cos(theta) * gap; 
     var y = y0 + theta * Math.sin(theta) * gap; 
     ctx.lineTo(x, y);
     theta += increment;
  }
  ctx.strokeStyle = col;
  ctx.stroke(); // draw the spiral
  ctx.strokeStyle= "#000000";   // restore to black  
}


/* draw 1/4 circle (circle quarter) 
for parameters see: https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D/arc
https://www.w3schools.com/tags/canvas_arc.asp
  j = 0..3   => circle segment
*/
function quarter(ctx, x, y, r, j, col)
{
  if (j < 0 && j >3) return;
  ctx.beginPath();
  if ( j === 0)  {  //1/4 circle segment UP-RIGHT oriented 
    ctx.arc(x, y, r, 0, 1.5*Math.PI, true);   //  	context.arc(x,y,r,sAngle,eAngle,counterclockwise);
  } 
  else if (j === 1)  {  //1/4 circle segment DOWN-RIGHT oriented 
    ctx.arc(x, y, r, 0, 0.5*Math.PI);   
  }
  else if (j === 2)  {  //1/4 circle segment DOWN-LEFT oriented 
    ctx.arc(x, y, r, 0.5*Math.PI, Math.PI);   
  } 
  else {                      //1/4 circle segment UP-LEFT oriented 
    ctx.arc(x, y, r, Math.PI, 1.5*Math.PI);   
  }  
  ctx.lineTo(x, y);
  ctx.closePath();
  ctx.fillStyle= col;                 
  ctx.fill(); 
}

/*
  ctx = canvas (object) context,
  x = x-center,
  y= y-center,
  r = radius,  
 if (happy == true)  => draw a happy smiley else draw a sad smiley,
 col = color      
*/ 
function smiley(ctx, x, y, r, happy, col)   
{
  //var old_w= ctx.lineWidth;     
   ctx.beginPath();          
   ctx.strokeStyle= col; 
   
   ctx.arc(x, y, r, 0, 2*Math.PI);  	// outer border    
   ctx.stroke();     
      
   ctx.beginPath();    
   ctx.arc(x-r/2, y-r/2, r/5, 0, 2*Math.PI);    // left eye
   ctx.stroke();  
    
   ctx.beginPath();    
   ctx.arc(x+r/2, y-r/2, r/5, 0, 2*Math.PI);    // right eye   
   ctx.stroke();  
      
   ctx.beginPath();          
   happy ? ctx.arc(x, y, r/2, 0, Math.PI) : ctx.arc(x, y+r/2, r/2, Math.PI, 0);  // mouth (happy or sad)
   ctx.stroke();
    
   ctx.strokeStyle= "#000000";   // restore to black
   //ctx.lineWidth= old_w; // restore old line width   
}
